@@ -818,7 +818,7 @@ def nb_filters_classes(weights_path):
 
 
 #####################
-# Vizualization #
+# Visualization #
 #####################
 
 def plot_losses(train_loss, test_loss):
@@ -1011,6 +1011,24 @@ def plot_trajectories_transitions(trans_dict, k, plot_values=False, **kwargs):
     return
 
 
+def animation_from_png(png_dir, moviename='anim', duration=1,
+                       savedir='./', remove_dir=True):
+    """
+    Create animation from saved png files
+    """
+    import os, imageio, shutil
+    images = []
+    if ".ipynb_checkpoints" in os.listdir(png_dir):
+        shutil.rmtree(os.path.join(png_dir, ".ipynb_checkpoints"))
+    for file_name in sorted(os.listdir(png_dir),
+                            key=lambda fname: int(fname.split('.')[0])):
+        if file_name.endswith('.png'):
+            images.append(imageio.imread(os.path.join(png_dir, file_name)))
+    imageio.mimsave(os.path.join(savedir, moviename + '.gif'), images)
+    if remove_dir:
+        shutil.rmtree(png_dir)
+
+
 #############################
 # Training data preparation #
 #############################
@@ -1193,13 +1211,17 @@ def extract_subimages(imgdata, coordinates, window_size, coord_class=0):
     (usually from a neural network output)
 
     Args:
-        imgdata (numpy array): 4D stack of images (n, height, width, channel)
-        coordinates (dict): Prediction from atomnet.locator
+        imgdata (numpy array):
+            4D stack of images (n, height, width, channel).
+            It is also possible to pass a single 2D image.
+        coordinates (dict or N x 2 numpy arry): Prediction from atomnet.locator
             (can be from other source but must be in the same format)
             Each element is a :math:`N \\times 3` numpy array,
             where *N* is a number of detected atoms/defects,
             the first 2 columns are *xy* coordinates
-            and the third columns is class (starts with 0)
+            and the third columns is class (starts with 0).
+            It is also possible to pass N x 2 numpy array if the corresponding
+            imgdata is a single 2D image.
         window_size (int):
             Side of the square for subimage cropping
         coord_class (int):
@@ -1212,6 +1234,12 @@ def extract_subimages(imgdata, coordinates, window_size, coord_class=0):
         ii) (x, y) coordinates of their centers,
         iii) frame number associated with each subimage
     """
+    if isinstance(coordinates, np.ndarray):
+        coordinates = np.concatenate((
+            coordinates, np.zeros((coordinates.shape[0], 1))), axis=-1)
+        coordinates = {0: coordinates}
+    if np.ndim(imgdata) == 2:
+        imgdata = imgdata[None, ..., None]
     subimages_all, com_all, frames_all = [], [], []
     for i, (img, coord) in enumerate(
             zip(imgdata, coordinates.values())):
@@ -1222,9 +1250,10 @@ def extract_subimages(imgdata, coordinates, window_size, coord_class=0):
         subimages_all.append(stack_i)
         com_all.append(com_i)
         frames_all.append(np.ones(len(com_i), int) * i)
-    subimages_all = np.concatenate(subimages_all, axis=0)
-    com_all = np.concatenate(com_all, axis=0)
-    frames_all = np.concatenate(frames_all, axis=0)
+    if len(subimages_all) > 0:
+        subimages_all = np.concatenate(subimages_all, axis=0)
+        com_all = np.concatenate(com_all, axis=0)
+        frames_all = np.concatenate(frames_all, axis=0)
     return subimages_all, com_all, frames_all
 
 
@@ -1959,3 +1988,53 @@ def threshImg(diff, threshL=0.25, threshH=0.75):
     threshIH = diff > threshH
     threshI = threshIL + threshIH
     return threshI
+
+
+def crop_borders(imgdata, thresh=0):
+    """
+    Crops image border where all values are zeros
+
+    Args:
+        imgdata (numpy array): 3D numpy array (h, w, c)
+        thresh: border values to crop
+
+    Returns: Cropped array
+    """
+    def crop(img):
+        mask = img > thresh
+        img = img[np.ix_(mask.any(1), mask.any(0))]
+        return img
+
+    imgdata_cr = [crop(imgdata[..., i]) for i in range(imgdata.shape[-1])]
+
+    return np.array(imgdata_cr).transpose(1, 2, 0)
+
+
+def get_coord_grid(imgdata, step, return_dict=True):
+    """
+    Generate a square coordinate grid for every image in a stack. Returns coordinates
+    in a dictionary format (same format as generated by atomnet.predictor)
+    that can be used as an input for utility functions extracting subimages
+    and atomstat.imlocal class
+
+    Args:
+        imgdata (numpy array): 2D or 3D numpy array
+        step (int): distance between grid points
+        return_dict (bool): returns coordiantes as a dictionary (same format as atomnet.predictor)
+
+    Returns:
+        Dictionary or numpy array with coordinates
+    """
+    if np.ndim(imgdata) == 2:
+        imgdata = np.expand_dims(imgdata, axis=0)
+    coord = []
+    for i in range(0, imgdata.shape[1], step):
+        for j in range(0, imgdata.shape[2], step):
+            coord.append(np.array([i, j]))
+    coord = np.array(coord)
+    if return_dict:
+        coord = np.concatenate((coord, np.zeros((coord.shape[0], 1))), axis=-1)
+        coordinates_dict = {i: coord for i in range(imgdata.shape[0])}
+        return coordinates_dict
+    coordinates = [coord for _ in range(imgdata.shape[0])]
+    return np.concatenate(coordinates, axis=0)
